"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * A Component can be thought of as a collection of data
 * which is attached to an entity.
 */
class Component {
    constructor(type, entity_id) {
        this.type = type;
        this.entity_id = entity_id;
        this.active = true;
        if (this.on_initialize !== undefined) {
            this.on_initialize();
        }
    }
    /**
     * @internal
     *
     * Activates the component, making it visible to engines..
     * Components are active by default when added to an entity.
     */
    activate() {
        if (!this.active) {
            if (this.on_activate !== undefined) {
                this.on_activate();
            }
            this.active = true;
        }
    }
    /**
     * @internal
     *
     * Deactivates the component, making it invisible to engines.
     */
    deactivate() {
        if (this.active) {
            if (this.on_deactivate !== undefined) {
                this.on_deactivate();
            }
            this.active = false;
        }
    }
    /**
     * Destroys the component and triggers the on_destroy callback.
     * Calling this function from outside the framework is a mistake.
     */
    destroy() {
        if (this.on_destroy !== undefined) {
            this.on_destroy();
        }
    }
}
exports.Component = Component;
