"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const encompass_gc_optimized_collections_1 = require("encompass-gc-optimized-collections");
class ComponentManager {
    constructor() {
        this.component_type_to_active_components_map = new encompass_gc_optimized_collections_1.GCOptimizedMap();
        this.component_type_to_inactive_components_map = new encompass_gc_optimized_collections_1.GCOptimizedMap();
        this.components_to_activate = new encompass_gc_optimized_collections_1.GCOptimizedList();
        this.components_to_deactivate = new encompass_gc_optimized_collections_1.GCOptimizedList();
        this.components_to_remove = new encompass_gc_optimized_collections_1.GCOptimizedList();
    }
    create_component(ComponentType, entity_id, draw_layer_manager, properties) {
        const component = new ComponentType(ComponentType, entity_id, draw_layer_manager);
        if (!this.component_type_to_active_components_map.has(ComponentType)) {
            this.component_type_to_active_components_map.set(ComponentType, new encompass_gc_optimized_collections_1.GCOptimizedList());
            this.component_type_to_inactive_components_map.set(ComponentType, new encompass_gc_optimized_collections_1.GCOptimizedList());
        }
        this.mark_for_activation(component);
        // initializing the properties
        if (properties !== undefined) {
            for (const [key, value] of Object.entries(properties)) {
                component[key] = value;
            }
        }
        return component;
    }
    get_components_by_component_type(ComponentType) {
        if (this.component_type_to_active_components_map.has(ComponentType)) {
            return this.component_type_to_active_components_map.get(ComponentType);
        }
        else {
            return encompass_gc_optimized_collections_1.GCOptimizedList.Empty;
        }
    }
    get_inactive_components_by_component_type(ComponentType) {
        if (this.component_type_to_inactive_components_map.has(ComponentType)) {
            return this.component_type_to_inactive_components_map.get(ComponentType);
        }
        else {
            return encompass_gc_optimized_collections_1.GCOptimizedList.Empty;
        }
    }
    activate_components() {
        for (const component of this.components_to_activate.values()) {
            this.perform_activate_component(component);
        }
        this.components_to_activate.clear();
    }
    deactivate_components() {
        for (const component of this.components_to_deactivate.values()) {
            this.perform_deactivate_component(component);
        }
        this.components_to_deactivate.clear();
    }
    remove_components() {
        for (const component of this.components_to_remove.values()) {
            this.perform_remove_component(component);
        }
        this.components_to_remove.clear();
    }
    mark_for_activation(component) {
        this.components_to_activate.add(component);
    }
    mark_for_deactivation(component) {
        this.components_to_deactivate.add(component);
    }
    mark_for_removal(component) {
        this.components_to_remove.add(component);
    }
    perform_remove_component(component) {
        this.component_type_to_active_components_map.get(component.type).deleteValue(component);
        this.component_type_to_inactive_components_map.get(component.type).deleteValue(component);
    }
    perform_deactivate_component(component) {
        this.component_type_to_active_components_map.get(component.type).deleteValue(component);
        this.component_type_to_inactive_components_map.get(component.type).add(component);
    }
    perform_activate_component(component) {
        this.component_type_to_inactive_components_map.get(component.type).deleteValue(component);
        this.component_type_to_active_components_map.get(component.type).add(component);
    }
}
exports.ComponentManager = ComponentManager;
