"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const engine_1 = require("../engine");
const entity_checker_1 = require("./utility/entity_checker");
const entity_tracker_1 = require("./utility/entity_tracker");
/**
 * A Detector is an Engine which is responsible
 * for reading the game world and producing Messages
 * in response.
 *
 * Detector are defined by the Component types they
 * track and the detect function they implement.
 *
 * It is an anti-pattern to modify Entities or
 * Components from inside a Detector.
 */
class Detector extends engine_1.Engine {
    constructor() {
        super(...arguments);
        this.entity_tracker = new entity_tracker_1.EntityTracker();
    }
    update(dt) {
        for (const id of this.entity_tracker.tracked_ids.entries()) {
            const entity = this.get_entity(id);
            this.detect(entity, dt);
        }
    }
    check_and_track_entity(entity_id) {
        const entity = this.get_entity(entity_id);
        const should_track = entity_checker_1.EntityChecker.check_entity(entity, this.component_types);
        if (should_track) {
            this.entity_tracker.track_entity(entity_id);
        }
        return should_track;
    }
    check_and_untrack_entity(entity_id) {
        const entity = this.get_entity(entity_id);
        const should_untrack = entity === undefined || !entity_checker_1.EntityChecker.check_entity(entity, this.component_types);
        if (should_untrack) {
            this.entity_tracker.untrack_entity(entity_id);
        }
        return should_untrack;
    }
    is_tracking(id) {
        return this.entity_tracker.tracked_ids.has(id);
    }
}
exports.Detector = Detector;
