import { GCOptimizedList } from "encompass-gc-optimized-collections";
import { Engine } from "../../engine";
import { Entity } from "../../entity";
import { EntityMessage } from "../../types/entity_message";
/**
 * An EntityModifier is an Engine which is responsible
 * for consuming EntityMessages and modifying Entities
 * in response.
 *
 * EntityModifiers are defined by their message_type
 * and the modify function they implement.
 *
 * It is an anti-pattern to read data from Entities
 * or Components inside an EntityModifier.
 */
export declare abstract class EntityModifier extends Engine {
    private entity_id_to_messages;
    update(dt: number): void;
    protected clean(): void;
    protected abstract modify(entity: Entity, messages: GCOptimizedList<EntityMessage>, dt: number): void;
    private retrieve_and_map_messages_by_entity_id;
}
