"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const encompass_gc_optimized_collections_1 = require("encompass-gc-optimized-collections");
class EntityTracker {
    constructor() {
        this.tracked_ids = new encompass_gc_optimized_collections_1.GCOptimizedSet();
        this.deactivated_ids = new encompass_gc_optimized_collections_1.GCOptimizedSet();
    }
    track_entity(id) {
        if (this.tracked_ids.has(id)) {
            return;
        }
        this.tracked_ids.add(id);
    }
    untrack_entity(id) {
        if (!this.tracked_ids.has(id)) {
            return;
        }
        this.tracked_ids.delete(id);
        this.deactivated_ids.delete(id);
    }
    on_entity_activated(id) {
        this.deactivated_ids.add(id);
        this.tracked_ids.delete(id);
    }
    on_entity_deactivated(id) {
        this.deactivated_ids.delete(id);
        this.tracked_ids.add(id);
    }
}
exports.EntityTracker = EntityTracker;
