"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class PriorityQueue {
    constructor() {
        this.items = new Map();
        this.priority_map = new Map();
        this._size = 0;
    }
    get size() {
        return this._size;
    }
    add(value, priority) {
        for (const [index, v] of this.items) {
            if (priority < this.priority_map.get(v)) {
                this.insert(index, value, priority);
                return;
            }
        }
        this.insert(this._size, value, priority);
    }
    find_min() {
        return this.items.get(0);
    }
    delete_min() {
        const min = this.items.get(0);
        this.delete(0);
        return min;
    }
    empty() {
        return this.size === 0;
    }
    /**
     * Iterates from lowest to highest.
     */
    to_array() {
        const entry_array = [];
        for (const entry of this.items.entries()) {
            entry_array.push(entry);
        }
        const sorted_array = entry_array.sort((a, b) => a[0] - b[0]);
        const values = [];
        for (const entry of sorted_array) {
            values.push(entry[1]);
        }
        return values;
    }
    delete(index) {
        if (this.items.has(index)) {
            const value = this.items.get(index);
            this.items.delete(index);
            this.priority_map.delete(value);
            let k = index;
            this._size -= 1;
            while (k < this.size) {
                const one_up_value = this.items.get(k + 1);
                if (one_up_value !== undefined) {
                    this.items.set(k, one_up_value);
                }
                k += 1;
            }
            this.items.delete(this.size);
        }
    }
    insert(index, value, priority) {
        let k = this._size;
        while (k > index) {
            this.items.set(k, this.items.get(k - 1));
            k -= 1;
        }
        this.items.set(index, value);
        this.priority_map.set(value, priority);
        this._size += 1;
    }
}
exports.PriorityQueue = PriorityQueue;
