"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const encompass_gc_optimized_collections_1 = require("encompass-gc-optimized-collections");
const message_pool_1 = require("./message_pool");
const message_query_1 = require("./message_query");
class MessageManager {
    constructor() {
        this.message_query = new message_query_1.MessageQuery(); // delegate through?
        this.messages = new encompass_gc_optimized_collections_1.GCOptimizedSet();
        this.message_pools = new encompass_gc_optimized_collections_1.GCOptimizedMap();
        this.message_type_to_messages_map = new encompass_gc_optimized_collections_1.GCOptimizedMap();
    }
    create_component_message(ComponentMessageType, component) {
        const component_message = this.create_or_get_message_pool(ComponentMessageType).obtain();
        component_message.component = component;
        this.messages.add(component_message);
        this.cache_message_by_message_type(component_message);
        return component_message;
    }
    create_entity_message(EntityMessageType, entity) {
        const entity_message = this.create_or_get_message_pool(EntityMessageType).obtain();
        entity_message.entity = entity;
        this.messages.add(entity_message);
        this.cache_message_by_message_type(entity_message);
        return entity_message;
    }
    create_message(MessageType) {
        const message = this.create_or_get_message_pool(MessageType).obtain();
        this.messages.add(message);
        this.cache_message_by_message_type(message);
        return message;
    }
    clear_messages() {
        for (const [_, message_set] of this.message_type_to_messages_map.entries()) {
            message_set.clear();
        }
        for (const message of this.messages.entries()) {
            this.message_pools.get(message.type).free(message);
        }
        this.messages.clear();
    }
    get_messages_by_message_type(MessageType) {
        if (this.message_type_to_messages_map.has(MessageType)) {
            return this.message_type_to_messages_map.get(MessageType);
        }
        else {
            return encompass_gc_optimized_collections_1.GCOptimizedList.Empty;
        }
    }
    cache_message_by_message_type(message) {
        if (!this.message_type_to_messages_map.get(message.type)) {
            this.message_type_to_messages_map.set(message.type, new encompass_gc_optimized_collections_1.GCOptimizedList());
        }
        this.message_type_to_messages_map.get(message.type).add(message);
    }
    create_or_get_message_pool(MessageType) {
        if (!this.message_pools.has(MessageType)) {
            this.message_pools.set(MessageType, new message_pool_1.MessagePool(MessageType));
        }
        return this.message_pools.get(MessageType);
    }
}
exports.MessageManager = MessageManager;
