"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const encompass_gc_optimized_collections_1 = require("encompass-gc-optimized-collections");
class ObjectPool {
    constructor(object_constructor, object_destructor = ObjectPool.DefaultDestructor, size = 16, overflow_behavior = ObjectPool.OverflowBehaviorExpand) {
        this._size = size;
        this._inactive_objects = new encompass_gc_optimized_collections_1.GCOptimizedList();
        this._active_objects = new encompass_gc_optimized_collections_1.GCOptimizedList();
        this.reference_object = new object_constructor(object_constructor);
        this.constructor_function = object_constructor;
        this.destructor_function = object_destructor;
        this.overflow_behavior = overflow_behavior;
        this.generate(size);
    }
    get size() { return this._size; }
    get inactive_objects() { return this._inactive_objects; }
    get active_objects() { return this._active_objects; }
    obtain() {
        const object = this.get_inactive_object();
        if (object !== null) {
            for (const key in object) {
                if (key !== "_type" && key !== "type") {
                    object[key] = this.reference_object[key];
                }
            }
            this._active_objects.add(object);
        }
        return object;
    }
    free(instance) {
        if (this.active_objects.deleteValue(instance)) {
            this.inactive_objects.add(instance);
            this.destructor_function(instance);
        }
        else {
            throw ("cannot free object which is not from pool");
        }
    }
    free_all() {
        for (const [_, object] of this.active_objects.entries()) {
            this.free(object);
        }
    }
    generate(amount) {
        let i = 0;
        while (i < amount) {
            const object = new this.constructor_function(this.constructor_function);
            this._inactive_objects.add(object);
            i += 1;
        }
    }
    get_inactive_object() {
        if (this._inactive_objects.size > 0) {
            return this._inactive_objects.pop();
        }
        else {
            return this.overflow_behavior(this);
        }
    }
}
ObjectPool.OverflowBehaviorFallible = () => null;
ObjectPool.OverflowBehaviorCycle = (pool) => {
    return pool.active_objects.shift();
};
ObjectPool.OverflowBehaviorExpand = (pool) => {
    pool.generate(pool.size);
    pool._size *= 2;
    return pool._inactive_objects.pop();
};
ObjectPool.DefaultDestructor = (object) => {
    // nullify the properties
    for (const key in object) {
        if (key !== "_type" && key !== "type") {
            object[key] = undefined;
        }
    }
};
exports.ObjectPool = ObjectPool;
