"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const encompass_gc_optimized_collections_1 = require("encompass-gc-optimized-collections");
const draw_layer_manager_1 = require("./draw_layer_manager");
const type_guards_1 = require("./type_guards");
class RenderManager {
    constructor(entity_to_id_map) {
        this.draw_component_type_to_renderer = new encompass_gc_optimized_collections_1.GCOptimizedMap();
        this.layer_index_to_drawable_set = new encompass_gc_optimized_collections_1.GCOptimizedMap();
        this.order = new encompass_gc_optimized_collections_1.GCOptimizedSortedList();
        this.entity_to_id_map = entity_to_id_map;
        this.draw_layer_manager = new draw_layer_manager_1.DrawLayerManager(this.layer_index_to_drawable_set, this.order);
    }
    register_entity_renderer(renderer) {
        this.draw_component_type_to_renderer.set(renderer.draw_component_type, renderer);
    }
    draw() {
        for (const [_, layer_index] of this.order.entries()) {
            for (const drawable of this.layer_index_to_drawable_set.get(layer_index).entries()) {
                if (type_guards_1.TypeGuard.drawable_is_general_renderer(drawable)) {
                    drawable.render();
                }
                else if (type_guards_1.TypeGuard.drawable_is_draw_component(drawable)) {
                    if (this.draw_component_type_to_renderer.has(drawable.type)) {
                        const renderer = this.draw_component_type_to_renderer.get(drawable.type);
                        const entity = this.entity_to_id_map.get(drawable.entity_id);
                        if (renderer.is_tracking(entity.id)) {
                            renderer.render(entity);
                        }
                    }
                }
            }
        }
    }
}
exports.RenderManager = RenderManager;
