"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const singleton_read_1 = require("./errors/singleton_read");
class Renderer {
    constructor(entity_manager, component_manager) {
        this.entity_manager = entity_manager;
        this.component_manager = component_manager;
    }
    get_entity(entity_id) {
        return this.entity_manager.get_entity(entity_id);
    }
    /**
     * Gets all active components of ComponentType.
     * The components are marked as readonly. For mutable components,
     * use get_mutable_components_by_type.
     *
     * @param ComponentType
     */
    read_components(ComponentType) {
        return this.component_manager.get_components_by_component_type(ComponentType);
    }
    /**
     * A convenience method for the case of singleton Components.
     *
     * @param ComponentType
     * @throws If more than one component of ComponentType exists
     * @returns A component of type ComponentType
     */
    read_component(ComponentType) {
        const components = this.component_manager.get_components_by_component_type(ComponentType);
        if (components.size === 0) {
            return null;
        }
        else if (components.size === 1) {
            return components.get(0);
        }
        else {
            const multiple_component_error = new singleton_read_1.SingletonReadError(this.constructor.name, ComponentType);
            throw multiple_component_error.toString();
        }
    }
}
exports.Renderer = Renderer;
