"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const entity_checker_1 = require("../engines/utility/entity_checker");
const entity_tracker_1 = require("../engines/utility/entity_tracker");
const renderer_1 = require("../renderer");
/**
 * An EntityRenderer is an Engine which is responsible for
 * reading Entities with specific Components to draw elements to the screen.
 *
 * EntityRenderers are defined by the Component types they
 * track and by their specific draw_component_type.
 * They also implement a render function.
 *
 * It is an anti-pattern to modify Entities or
 * Components from within a EntityRenderer.
 */
class EntityRenderer extends renderer_1.Renderer {
    constructor() {
        super(...arguments);
        this.component_types = [];
        this.entity_tracker = new entity_tracker_1.EntityTracker();
    }
    check_and_track_entity(entity_id) {
        const entity = this.get_entity(entity_id);
        const should_track = this.check_entity(entity);
        if (should_track) {
            this.entity_tracker.track_entity(entity_id);
        }
        return should_track;
    }
    check_and_untrack_entity(entity_id) {
        const entity = this.get_entity(entity_id);
        const should_untrack = entity === undefined || !this.check_entity(entity);
        if (should_untrack) {
            this.entity_tracker.untrack_entity(entity_id);
        }
        return should_untrack;
    }
    is_tracking(id) {
        return this.entity_tracker.tracked_ids.has(id);
    }
    check_entity(entity) {
        return entity_checker_1.EntityChecker.check_entity(entity, this.component_types) &&
            entity.has_component(this.draw_component_type);
    }
}
exports.EntityRenderer = EntityRenderer;
