import { GCOptimizedList } from "encompass-gc-optimized-collections";
import { ComponentManager } from "./component_manager";
import { Engine } from "./engine";
import { EntityCreator } from "./entity_creator";
import { EntityManager } from "./entity_manager";
import { MessageManager } from "./message_manager";
import { RenderManager } from "./render_manager";
/**
 * A World is the glue that holds all the framework
 * elements together.
 *
 * It is a mistake to instantiate elements of encompass
 * separately from a World. These elements should be
 * created either through appropriate World functions,
 * or by Engines.
 *
 * It is perfectly valid to use multiple Worlds in
 * the same game. For example, one World could
 * drive the main menu, and another could drive
 * the game logic.
 */
export declare class World {
    engine_order: GCOptimizedList<Engine>;
    private component_manager;
    private entity_creator;
    private entity_manager;
    private message_manager;
    private render_manager;
    constructor(component_manager: ComponentManager, entity_creator: EntityCreator, entity_manager: EntityManager, message_manager: MessageManager, render_manager: RenderManager);
    /**
     * Updates the simulation based on given delta time,
     * advancing the simulation by one frame.
     *
     * @param dt Delta time
     */
    update(dt: number): void;
    /**
     * Calls the render methods of all Renderers in
     * the World ordered by the layers on DrawComponents.
     */
    draw(): void;
    private destroy_marked_entities;
    private process;
}
