"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const jprof = __importStar(require("encompass-jprof"));
/**
 * A World is the glue that holds all the framework
 * elements together.
 *
 * It is a mistake to instantiate elements of encompass
 * separately from a World. These elements should be
 * created either through appropriate World functions,
 * or by Engines.
 *
 * It is perfectly valid to use multiple Worlds in
 * the same game. For example, one World could
 * drive the main menu, and another could drive
 * the game logic.
 */
class World {
    constructor(component_manager, entity_creator, entity_manager, message_manager, render_manager) {
        this.component_manager = component_manager;
        this.entity_creator = entity_creator;
        this.entity_manager = entity_manager;
        this.message_manager = message_manager;
        this.render_manager = render_manager;
    }
    /**
     * Updates the simulation based on given delta time,
     * advancing the simulation by one frame.
     *
     * @param dt Delta time
     */
    update(dt) {
        jprof.push("frame");
        jprof.push("World.update");
        jprof.push("process");
        this.process(dt);
        jprof.pop("process");
        jprof.push("World.message_manager.clear_messages");
        this.message_manager.clear_messages();
        jprof.pop("World.message_manager.clear_messages");
        jprof.push("World.destroy_marked_entities");
        this.destroy_marked_entities();
        jprof.pop("World.destroy_marked_entities");
        jprof.push("World.entity_manager.check_entities_with_added_components");
        this.entity_manager.check_entities_with_added_components();
        jprof.pop("World.entity_manager.check_entities_with_added_components");
        jprof.push("World.entity_manager.check_entities_with_removed_components");
        this.entity_manager.check_entities_with_removed_components();
        jprof.pop("World.entity_manager.check_entities_with_removed_components");
        jprof.push("World.component_manager.activate_components");
        this.component_manager.activate_components();
        jprof.pop("World.component_manager.activate_components");
        jprof.push("World.component_manager.deactivate_components");
        this.component_manager.deactivate_components();
        jprof.pop("World.component_manager.deactivate_components");
        jprof.push("World.component_manager.remove_components");
        this.component_manager.remove_components();
        jprof.pop("World.component_manager.remove_components");
        jprof.pop("World.update");
    }
    /**
     * Calls the render methods of all Renderers in
     * the World ordered by the layers on DrawComponents.
     */
    draw() {
        jprof.push("World.draw");
        this.render_manager.draw();
        jprof.pop("World.draw");
        jprof.pop("frame");
    }
    destroy_marked_entities() {
        this.entity_manager.destroy_marked_entities();
    }
    process(dt) {
        for (const engine of this.engine_order.values()) {
            jprof.push(engine.constructor.name + ".update");
            engine.update(dt);
            jprof.pop(engine.constructor.name + ".update");
        }
    }
}
exports.World = World;
