import { Type } from "./types/generic_type";
/**
 * A Component can be thought of as a collection of data
 * which is attached to an entity.
 */
export declare abstract class Component {
    readonly type: Type<Component>;
    active: boolean;
    entity_id: number;
    constructor(type: Type<Component>, entity_id: number);
    /**
     * @internal
     *
     * Activates the component, making it visible to engines..
     * Components are active by default when added to an entity.
     */
    activate(): void;
    /**
     * @internal
     *
     * Deactivates the component, making it invisible to engines.
     */
    deactivate(): void;
    /**
     * Destroys the component and triggers the on_destroy callback.
     * Calling this function from outside the framework is a mistake.
     */
    destroy(): void;
    protected on_initialize?(): void;
    protected on_activate?(): void;
    protected on_deactivate?(): void;
    protected on_destroy?(): void;
}
