import { Component } from "../component";
import { Engine } from "../engine";
import { Entity } from "../entity";
import { IEntityTracker } from "../interfaces/entity_tracker";
import { Type } from "../types/generic_type";
/**
 * A Detector is an Engine which is responsible
 * for reading the game world and producing Messages
 * in response.
 *
 * Detector are defined by the Component types they
 * track and the detect function they implement.
 *
 * It is an anti-pattern to modify Entities or
 * Components from inside a Detector.
 */
export declare abstract class Detector extends Engine implements IEntityTracker {
    readonly component_types: Type<Component>[];
    private entity_tracker;
    update(dt: number): void;
    check_and_track_entity(entity_id: number): boolean;
    check_and_untrack_entity(entity_id: number): boolean;
    is_tracking(id: number): boolean;
    /**
     * Each World update, this callback runs for each
     * entity tracked by the Detector.
     *
     * @param entity An entity that is being tracked
     * by the Detector.
     */
    protected abstract detect(entity: Entity, dt: number): void;
}
