import { GCOptimizedList } from "encompass-gc-optimized-collections";
import { Component } from "../../component";
import { Engine } from "../../engine";
import { ComponentMessage } from "../../types/component_message";
import { Mutable } from "../../types/mutable";
/**
 * A ComponentModifier is an Engine which is responsible
 * for consuming ComponentMessages and modifying the referenced
 * Component in response.
 *
 * ComponentModifiers are defined by their message_type
 * and the modify function they implement.
 *
 * It is an anti-pattern to read Entities or Components
 * within a ComponentModifier.
 */
export declare abstract class ComponentModifier extends Engine {
    private component_to_messages;
    update(dt: number): void;
    protected abstract modify(component: Mutable<Component>, messages: GCOptimizedList<ComponentMessage>, dt: number): void;
    protected clean(): void;
    private retrieve_and_map_messages_by_component_instance;
}
