import { GCOptimizedList } from "encompass-gc-optimized-collections";
import { Component } from "./component";
import { ComponentManager } from "./component_manager";
import { EntityManager } from "./entity_manager";
import { Type } from "./types/generic_type";
export declare abstract class Renderer {
    private entity_manager;
    private component_manager;
    constructor(entity_manager: EntityManager, component_manager: ComponentManager);
    protected get_entity(entity_id: number): import("./entity").Entity | undefined;
    /**
     * Gets all active components of ComponentType.
     * The components are marked as readonly. For mutable components,
     * use get_mutable_components_by_type.
     *
     * @param ComponentType
     */
    protected read_components<TComponent extends Component>(ComponentType: Type<TComponent>): GCOptimizedList<Readonly<TComponent>>;
    /**
     * A convenience method for the case of singleton Components.
     *
     * @param ComponentType
     * @throws If more than one component of ComponentType exists
     * @returns A component of type ComponentType
     */
    protected read_component<TComponent extends Component>(ComponentType: Type<TComponent>): Readonly<TComponent> | null;
}
