import { Component } from "../component";
import { DrawComponent } from "../draw_component";
import { Entity } from "../entity";
import { IDrawableEntityTracker } from "../interfaces/drawable_entity_tracker";
import { Renderer } from "../renderer";
import { Type } from "../types/generic_type";
/**
 * An EntityRenderer is an Engine which is responsible for
 * reading Entities with specific Components to draw elements to the screen.
 *
 * EntityRenderers are defined by the Component types they
 * track and by their specific draw_component_type.
 * They also implement a render function.
 *
 * It is an anti-pattern to modify Entities or
 * Components from within a EntityRenderer.
 */
export declare abstract class EntityRenderer extends Renderer implements IDrawableEntityTracker {
    readonly component_types: Type<Component>[];
    /**
     * A DrawComponent constructor.
     */
    readonly draw_component_type: Type<DrawComponent>;
    private entity_tracker;
    /**
     * Upon World draw, this function runs for each
     * entity tracked by the Renderer.
     *
     * @param entity A reference to an Entity which
     * is tracked by the Renderer.
     */
    abstract render(entity: Entity): void;
    check_and_track_entity(entity_id: number): boolean;
    check_and_untrack_entity(entity_id: number): boolean;
    is_tracking(id: number): boolean;
    protected check_entity(entity: Entity): boolean;
}
